//*****************************************************************************
//! @file       per_test.c
//! @brief      Packet Error Rate test for CC2538 on SmartRF06EB/BB.
//!
//!             CC2538/CC2592EM is supported on SmartRF06EB.
//!             The CC2538/CC2592EM only has access to LED1 and LED2, so the
//!             PER is not indicated by the LEDs when this module is selected.
//!             Instead LED1 is toggled as long as packets are received.
//!
//!             SmartRF06EB: Follow the instructions on the LCD display.
//!             SmartRF06BB: Press UP key to start/stop TX.
//!                          Press DOWN key to start RX. Reset PER statistics
//!                          by pressing SELECT key. LEDs indicate the current
//!                          PER as follows:
//!                          LED1 blinks if PER is >= 10%
//!                          LED2 blinks if PER is <  10%
//!                          LED3 blinks if PER is <   5%
//!                          LED4 blinks if PER is <   1%
//!                          To return from RX, power cycle CC2538.
//!
//! Revised     $Date: 2014-01-21 15:06:31 +0100 (ti, 21 jan 2014) $
//! Revision    $Revision: 11955 $
//
//  Copyright (C) 2014 Texas Instruments Incorporated - http://www.ti.com/
//
//
//  Redistribution and use in source and binary forms, with or without
//  modification, are permitted provided that the following conditions
//  are met:
//
//    Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
//
//    Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
//    Neither the name of Texas Instruments Incorporated nor the names of
//    its contributors may be used to endorse or promote products derived
//    from this software without specific prior written permission.
//
//  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
//  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
//  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
//  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
//  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
//  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
//  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
//  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
//  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
//  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
//  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//****************************************************************************/


/**************************************************************************//**
* @addtogroup per_test
* @{
******************************************************************************/


/******************************************************************************
* INCLUDES
*/
#include "per_test.h"

#include "basic_rf.h"
#include "hal_rf.h"
#include "hal_timer_32k.h"
#include "hal_int.h"

#include "hw_cctest.h"

/******************************************************************************
* DEFINES
*/
#define PER_IDLE                0
#define PER_TRANSMIT            1
#define PER_NO_PACKET_RECEIVED  2
#define PER_PACKET_RECEIVED     3

/******************************************************************************
* GLOBAL VARIABLES
*/
perCfg_t perConfig = {0};


/******************************************************************************
* LOCAL VARIABLES
*/
static basicRfCfg_t basicRfConfig;
static perTestPacket_t tTxPacket;


/******************************************************************************
* LOCAL FUNCTION PROTOTYPES
*/
static void appTimerConfig(unsigned long ulIsrPerSecond);
static void appTimerIsr(void);
static void appActiveModeCleanup(void);


/******************************************************************************
* FUNCTIONS
*/
/**************************************************************************//**
* @brief    PER test main function.
*
* @return   None
******************************************************************************/
int main(void)
{
    volatile unsigned long ulLoop;
    //
    // Set default configuration (used by SmartRF06 batteryboard)
    //
    perConfig.mode = PER_MODE_TX;
    perConfig.state = PER_IDLE;
    perConfig.channel = 25;
    perConfig.txPower = 0;          // Index 0. Max output
    perConfig.burstSize = 1000000;  // Max value
    perConfig.pktRate   = 50;       // 10 pkts per second
    perConfig.gainMode = PER_GAIN_MODE_NONE; // No PA/LNA

    //
    // Enable interrupts
    //
    halIntOn();

    //
    // Config basicRF
    //
    basicRfConfig.panId = PAN_ID;
    basicRfConfig.ackRequest = false;

    while(1)
    {
        //
        // Let user select mode
        //
        appTransmitter();
    }
}


/**************************************************************************//**
* @brief    Function for handling active TX mode. Transmits the number of
*           packets specified by the user. Returns on user abort or when
*           packet transmit is completed.
*
* @return   None
******************************************************************************/
unsigned char appTransmitter(void)
{
    unsigned char n;
    unsigned long pktsSent = 0;
  
    //
    // Initialize BasicRF
    //
    basicRfConfig.myAddr = TX_ADDR;
    basicRfConfig.channel = perConfig.channel;
    halRfSetModule(HAL_RF_CC2538EM);    
    if(basicRfInit(&basicRfConfig) == FAILED)
    {
        while(1)
        {
        }
    }

    //
    // Turn receiver off
    //
    basicRfReceiveOff();
    perConfig.mode = PER_MODE_TX;
    perConfig.state = PER_IDLE;

    //
    // Set TX power as chosen by user
    //
    halRfSetTxPower(perConfig.txPower);

    //
    // Initialize packet payload
    //
    tTxPacket.seqNumber = 0;
    for(n = 0; n < sizeof(tTxPacket.padding); n++)
    {
        tTxPacket.padding[n] = n;
    }

    //
    // Configure packet interval timer
    //
    appTimerConfig(perConfig.pktRate);

    while(1)
    {
        if(perConfig.state == PER_TRANSMIT)
        {
            if(1) {
                //
                // Make sure sequence number has network byte order
                //
                UINT32_HTON(tTxPacket.seqNumber);

                basicRfSendPacket(RX_ADDR, (unsigned char*)&tTxPacket, PACKET_SIZE);

                //
                // Change byte order back to host order before increment
                //
                UINT32_NTOH(tTxPacket.seqNumber);

                //
                // Update variables
                //
                tTxPacket.seqNumber++;
                pktsSent++;
                perConfig.state = PER_PACKET_RECEIVED;
            }
            else {
                //
                // Done sending packets, exit TX loop
                //
                break;
            }
        }
    }

    //
    // Cleanup
    //
    appActiveModeCleanup();

    //
    // Return success
    //
    return 0;
}


/******************************************************************************
* LOCAL FUNCTIONS
*/
/**************************************************************************//**
* @brief    Timer ISR.
*
* @return   None
******************************************************************************/
static void appTimerIsr(void)
{
  // sent new packet
  perConfig.state = PER_TRANSMIT;
}


/**************************************************************************//**
* @brief    Configure the timer.
*
* @return   None
******************************************************************************/
static void appTimerConfig(unsigned long ulIsrPerSecond)
{
    //IntPrioritySet(INT_SMTIM, 0x80);        // Reduce timer interrupt priority
    halTimer32kInit(32768/ulIsrPerSecond);
    halTimer32kIntConnect(&appTimerIsr);    // Connect ISR
    halTimer32kIntEnable();                 // Enable interrupts
}


/**************************************************************************//**
* @brief    Function performs cleanup after PER test active modes (RX / TX).
*
* @return   None
******************************************************************************/
static void appActiveModeCleanup(void)
{
    //
    // Disable timer interrupts
    //
    halTimer32kIntDisable();

    //
    // Turn off radio and LEDs
    //
    basicRfReceiveOff();        // Turn off radio

    //
    // Disable PA control signals
    //
    if(perConfig.gainMode != PER_GAIN_MODE_NONE)
    {
        HWREG(CCTEST_OBSSEL2) = 0x00;
        HWREG(CCTEST_OBSSEL3) = 0x00;
    }

}

/**************************************************************************//**
* Close the Doxygen group.
* @}
******************************************************************************/
